
// ****************************************************************************
// File: "Code Analyzer.h"
// Desc: Binary code hook analyze support
//
// ****************************************************************************
#pragma once

// "Hacker Disassembler Engine"
#include "hde32.h"

// Common x86 opcodes
const BYTE INSTR_TWOBYTE_0F			= 0x0F;	// Prefix for many two byte opcodes

const BYTE INSTR_FARJMP				= 0x2D;	// Far jmp prefixed with INSTR_FAR_PREFIX
const BYTE INSTR_REL8JCC_START		= 0x70;	// rel8 conditional jumps
const BYTE INSTR_REL8JCC_END		= 0x7F;
const BYTE INSTR_REL16_32JCC_START	= 0x80;	// rel16/32 conditional jumps prefixed w/INSTR_TWOBYTE_0F
const BYTE INSTR_REL16_32JCC_END	= 0x8F;
const BYTE INSTR_LOOPNZ				= 0xE0;
const BYTE INSTR_LOOPE				= 0xE1;
const BYTE INSTR_LOOP				= 0xE2;
const BYTE INSTR_REL8JCX			= 0xE3;
const BYTE INSTR_REL16_32JMP		= 0xE9;
const BYTE INSTR_REL8JMP			= 0xEB;

const BYTE INSTR_REL16_32CALL		= 0xE8;
const BYTE INSTR_RET				= 0xC2;
const BYTE INSTR_RETN				= 0xC3;
const BYTE INSTR_RETFN				= 0xCA;
const BYTE INSTR_RETF				= 0xCB;

const BYTE INSTR_INT3				= 0xCC;
const BYTE INSTR_NOP				= 0x90;


namespace CODE
{	
	// Return instruction relocation information for a given area of code space
	struct RELOCINFO
	{
		RELOCINFO()  { ZeroMemory(this, sizeof(RELOCINFO)); }
		~RELOCINFO() { Free(); }	
		void Free(){ if(pDissasm){ _aligned_free(pDissasm); } ZeroMemory(this, sizeof(RELOCINFO)); }
	
		UINT Flags;			// Flags (see below)		
		int  iInstCount;	// Target instruction relocation count
		int  iRoundedLen;	// Total rounded instruction bytes length
		LPHDE32S pDissasm;	// Linear array of dissasm structures; pDissasm[iInstCount]
	};

	const UINT HAS_INST_ERROR  = (1 << 0);	// If there was an instruction decode error
	const UINT HAS_REL_INST    = (1 << 1);	// Has a relative jump or call instruction
	const UINT WILL_OVERFLOW   = (1 << 2);	// Need to overwrite following alignment bytes for required space
	const UINT NOT_ENOUGH_ROOM = (1 << 3);	// Not enough room for requested relocation size

	BOOL GetRelocInfo(const PBYTE pCodeBytes, int iCodeLength, RELOCINFO &rInfo);
};