
/*
 * Hacker Disassembler Engine 32
 * Copyright (c) 2006-2009, Vyacheslav Patkov.
 * All rights reserved.
 * Mod by Sirmabus 2010
 *
 * hde32.h: C/C++ header file
 *
 */
#pragma once

#define F_MODRM         0x00000001  // ModR/M exists
#define F_SIB           0x00000002  // SIB exists
#define F_IMM8          0x00000004  // Immediate value imm8 exists
#define F_IMM16         0x00000008  // Immediate value imm16 exists
#define F_IMM32         0x00000010  // Immediate value imm32 exists
#define F_DISP8         0x00000020  // Displacement disp8 exists
#define F_DISP16        0x00000040  // Displacement disp16 exists
#define F_DISP32        0x00000080  // Displacement disp32 exists
#define F_RELATIVE      0x00000100  // Imm* is relative address
#define F_2IMM16        0x00000800

#define F_ERROR         0x00001000  // Invalid instruction
#define F_ERROR_OPCODE  0x00002000  // Invalid opcode
#define F_ERROR_LENGTH  0x00004000  // Length of command more than 15
#define F_ERROR_LOCK    0x00008000  // Prefix lock isn't allowed
#define F_ERROR_OPERAND 0x00010000  // Operand isn't allowed

#define F_PREFIX_REPNZ  0x01000000  // Repnz prefix exists
#define F_PREFIX_REPX   0x02000000  // Rep(z) prefix exists
#define F_PREFIX_REP    0x03000000  // Rep(z) or repnz prefix exists
#define F_PREFIX_66     0x04000000  // 0x66 prefix exists
#define F_PREFIX_67     0x08000000  // 0x67 prefix exists
#define F_PREFIX_LOCK   0x10000000  // Lock prefix exists
#define F_PREFIX_SEG    0x20000000  // Segment prefix exists
#define F_PREFIX_ANY    0x3f000000  // Any prefix esists

#define PREFIX_SEGMENT_CS   0x2E
#define PREFIX_SEGMENT_SS   0x36
#define PREFIX_SEGMENT_DS   0x3E
#define PREFIX_SEGMENT_ES   0x26
#define PREFIX_SEGMENT_FS   0x64
#define PREFIX_SEGMENT_GS   0x65
#define PREFIX_LOCK         0xF0
#define PREFIX_REPNZ        0xF2
#define PREFIX_REPX         0xF3
#define PREFIX_OPERAND_SIZE 0x66
#define PREFIX_ADDRESS_SIZE 0x67

#pragma pack(push,1)
struct HDE32S
{
	BYTE len;         // length of command
	BYTE p_rep;       // rep/repz (0xf3) & repnz (0xf2) prefix
	BYTE p_lock;      // lock prefix: 0xf0
	BYTE p_seg;       // segment prefix: 0x26,0x2e,0x36,0x3e,0x64,0x65
	BYTE p_66;        // operand-size override prefix: 0x66
	BYTE p_67;        // address-size override prefix: 0x67
	BYTE opcode;      // opcode
	BYTE opcode2;     // second opcode (if first opcode is 0x0f)
	BYTE modrm;       // ModR/M
	BYTE modrm_mod;   //   ModR/M.mod
	BYTE modrm_reg;   //   ModR/M.reg
	BYTE modrm_rm;    //   ModR/M.r/m
	BYTE sib;         // SIB
	BYTE sib_scale;   //   SIB.scale
	BYTE sib_index;   //   SIB.index
	BYTE sib_base;    //   SIB.base
	union
	{
		BYTE imm8;    // immediate value imm8
		WORD imm16;	  // immediate value imm16
		UINT imm32;   // immediate value imm32
	} imm;
	union
	{
		BYTE disp8;   // displacement disp8
		WORD disp16;  // displacement disp16
		UINT disp32;  // displacement disp32
	} disp;
	UINT flags;      // flags
};
#pragma pack(pop)
typedef HDE32S *LPHDE32S;

UINT hde32_disasm(LPCVOID code, LPHDE32S hs);
